/* ====================================================================
 *  Copyright 2004 Edward Rudd and Paul Querna
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *
 */

/**
 * mod_svn_view public API for sub-modules
 */

#include "httpd.h"
#include "svn_repos.h"
#include "svn_config.h"

#ifndef _MOD_SVN_VIEW_H_
#define _MOD_SVN_VIEW_H_

/**
 *  Structure to hold pointers and cached information about the SVN connection
 */
typedef struct svn_repo_rec svn_repo_rec;
struct svn_repo_rec
{
    /* SubPool for subversion allocations */
    apr_pool_t *p;
    request_rec *r;

    const char *sv_root;        /* Root URL for svn-view */

    const char *reponame;       /* Selected repository name */
    const char *repopath;       /* Selected repository filesystem path */
    const char *path;           /* Selected path/filename */
    const char *css;            /* Selected css uri */
    const char *xslt;           /* Selected xslt file */

    svn_revnum_t request_rev;   /* Selected revision */
    svn_node_kind_t kind;       /* Type of path (file or dir) */

    svn_repos_t *repo;          /* Repository handle */
    svn_fs_t *repo_fs;          /* Repository file system handle */
    svn_fs_root_t *rev_root;    /* file system revision root */
    svn_fs_dirent_t *dirent;    /* Repository directory list */
    svn_revnum_t last_rev;      /* The latest (ie. youngest) revision */

    const char* authz_file;     /* Private: Authz Filename for Access Checks */
    svn_config_t *authz;        /* Private: Authz data for Access Checks */
    apr_hash_t* repos;
};

typedef int sv_action_handler(svn_repo_rec * svn, apr_table_t * parms,
                              void *data);

/**
 * Handler states and flags
 */
enum {
    SV_FLAG_ACTION_NOHEAD = 0x01,
    SV_FLAG_ACTION_NORUN = 0x02,
};
/**
 * Register an action handler
 */

APR_DECLARE(void) sv_action_register(const char *action,
                                     sv_action_handler * func, int states,
                                     void *data, apr_pool_t * p);

/**
 * Authz Checking Functions
 */

APR_DECLARE( int ) sv_authz_check(svn_repo_rec * svn,
                                      const char* path_to_check);

APR_DECLARE( svn_error_t *) sv_authz_callback (svn_boolean_t *allowed,
                                              svn_fs_root_t *root,
                                              const char *path,
                                              void *baton,
                                              apr_pool_t *pool);
/**
 * Utility Functions
 */

APR_DECLARE( svn_error_t *) sv_generate_delta_tree(svn_repos_node_t ** tree,
    svn_repo_rec * svn,
    svn_revnum_t base_rev, svn_boolean_t use_copy_history);


#endif /* _MOD_SVN_VIEW_H_ */
